package com.thaiopensource.trex.util;

import org.apache.tools.ant.Task;
import org.apache.tools.ant.taskdefs.LogOutputStream;
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.types.FileSet;
import org.apache.tools.ant.DirectoryScanner;

import java.io.File;
import java.io.OutputStream;
import java.io.PrintStream;
import java.io.IOException;
import java.util.Vector;

import org.xml.sax.XMLReader;
import org.xml.sax.SAXParseException;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.ParserAdapter;

import com.thaiopensource.trex.PatternReader;
import com.thaiopensource.trex.PatternBuilder;
import com.thaiopensource.trex.Pattern;
import com.thaiopensource.trex.Validator;
import com.thaiopensource.trex.XMLReaderCreator;

import com.thaiopensource.datatype.xsd.DatatypeFactoryImpl;

/**
 * Ant task to validate XML files using TREX.
 */

public class TrexTask extends Task {

  private File trexFile;
  private File src;
  private Vector filesets = new Vector();

  public void execute() throws BuildException {
    if (trexFile == null)
      throw new BuildException("trexfile attribute must be set!", 
			       location);
    if (src == null && filesets.size() == 0)
      throw new BuildException("There must be a file attribute or a fileset child element",
			       location);

    ErrorHandlerImpl eh = new ErrorHandlerImpl(new PrintStream(new LogOutputStream(this, Project.MSG_WARN)));

    boolean hadError = false;

    try {
      XMLReaderCreator xrc = new Jaxp10XMLReaderCreator();
      
      XMLReader xr = xrc.createXMLReader();
      xr.setErrorHandler(eh);
      PatternBuilder pb = new PatternBuilder();
      Pattern p = PatternReader.readPattern(xrc, xr, pb,
					    new DatatypeFactoryImpl(),
					    fileInputSource(trexFile));
      if (p == null)
	hadError = true;
      else {
	if (src != null) {
	  if (!processFile(p, pb, xr, src))
	    hadError = true;
	}
	for (int i = 0; i < filesets.size(); i++) {
	  FileSet fs = (FileSet)filesets.elementAt(i);
	  DirectoryScanner ds = fs.getDirectoryScanner(project);
	  File dir = fs.getDir(project);
	  String[] srcs = ds.getIncludedFiles();
	  for (int j = 0; j < srcs.length; j++) {
	    if (!processFile(p, pb, xr, new File(dir, srcs[j])))
	      hadError = true;
	  }
	}
      }
    }
    catch (SAXException e) {
      hadError = true;
      eh.printException(e);
    }
    catch (IOException e) {
      hadError = true;
      eh.printException(e);
    }
    if (hadError)
      throw new BuildException("Validation failed, messages should have been provided.", location);
  }

  private boolean processFile(Pattern p,
			      PatternBuilder pb,
			      XMLReader xr,
			      File f) throws SAXException, IOException {
    Validator v = new Validator(p, pb, xr);
    xr.parse(fileInputSource(f));
    return v.getValid();
  }

  static private InputSource fileInputSource(File f) {
    return new InputSource(FileURL.fileToURL(f).toString());
  }

  public void setTrexfile(String trexFilename) {
    trexFile = project.resolveFile(trexFilename);
  }
    
  public void setFile(File file) {
    this.src = file;
  }
  
  public void addFileset(FileSet set) {
    filesets.addElement(set);
  }

}
